﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Reflection;
using System.ComponentModel;

namespace DeliciousDotNet.OfflineReader
{
	public class RecipeUpdater
	{
		public void LoadFromXml()
		{
			var xmlPath = @"http://delicious.net/recipes/";
			//var xmlPath = @"C:\DeliciousDotNet\DeliciousClient\data\recipes.xml";

			XDocument xmlDoc = XDocument.Load(xmlPath);
			DeliciousDataContext dataContext = new DeliciousDataContext();

            ClearDb(dataContext);

			foreach (var xmlRecipe in xmlDoc.Descendants("Recipe"))
			{
				Recipe recipe = new Recipe();
				dataContext.Recipes.Add(recipe);
				PopulateDomainObjectFromXmlNode(recipe, xmlRecipe);

				foreach (var xmlIngredient in xmlRecipe.Descendants("Ingredient"))
				{
					Ingredient ingredient = new Ingredient();
					recipe.Ingredients.Add(ingredient);
					PopulateDomainObjectFromXmlNode(ingredient, xmlIngredient);
				}

                foreach (var xmlDirection in xmlRecipe.Descendants("Direction"))
                {
                    Direction direction = new Direction();
                    recipe.Directions.Add(direction);
                    PopulateDomainObjectFromXmlNode(direction, xmlDirection);
                }
		
			}

			dataContext.SubmitChanges();
		}

		void PopulateDomainObjectFromXmlNode(object domainObject, XElement xmlNode)
		{
			var propertyNames = from p in domainObject.GetType().GetProperties()
													where !p.Name.Contains("ID") && IsDbColumn(p)
													select p.Name;

			foreach (string propertyName in propertyNames)
			{
				string valueFromXml = GetChildNodeValue(xmlNode, propertyName);
				domainObject.GetType().GetProperty(propertyName).SetValue(domainObject, valueFromXml, null);
			}
		}

		string GetChildNodeValue(XElement node, string childNodeName)
		{
			var kids = node.Descendants(childNodeName);
			return kids.Count() > 0 ? kids.First().Value : "";
		}

		bool IsDbColumn(PropertyInfo info)
		{
			return info.GetCustomAttributes(typeof(ColumnAttribute), true).Count() > 0;
		}

        void ClearDb(DeliciousDataContext dataContext)
        {
            if (!dataContext.DatabaseExists())
            {
                dataContext.CreateDatabase();
            }
            else
            {
                RemoveAll();
            }
        }

		public void RemoveAll()
		{
			DeliciousDataContext context = new DeliciousDataContext();
            context.ExecuteCommand("delete from Ingredient");
            context.ExecuteCommand("delete from Direction");
            context.ExecuteCommand("delete from Recipe");
		}
	}
}

#region Test
#if DEBUG

namespace DeliciousDotNet.OfflineReader.Testing
{
	using NUnit.Framework;

	[TestFixture()]
	public class RecipeUpdaterTest
	{
		[Test]
		public void TestRecipeLoadedCorrectly()
		{
			var recipe = context.Recipes.Where(r => r.Name == "Wiener Schnitzel").First();
			Assert.AreEqual("Crunchy, traditional Austrian schnitzel.", recipe.Description);
			Assert.AreEqual("beer", recipe.GoodWith);
			// etc

			Assert.AreEqual(9, recipe.Ingredients.Count);
			var ingredient = recipe.Ingredients.Where(i => i.Description == "milk").First();
			Assert.AreEqual("1/2", ingredient.Quantity);
			Assert.AreEqual("cup", ingredient.QuantityUnit);

			Assert.AreEqual(8, recipe.Directions.Count);
			var direction = recipe.Directions.Where(d => d.StepNumber == "5").First();
			Assert.AreEqual("Cook over low heat in 4 tablespoons butter until golden and tender.", direction.Description);
		}

		[Test]
		public void TestCorrectNumberOfRecordsCreated()
		{
			Assert.AreEqual(2, context.Recipes.Count());
			Assert.AreEqual(17, context.Ingredients.Count());
			Assert.AreEqual(13, context.Directions.Count());
		}

		[SetUp]
		public void Setup()
		{
			RecipeUpdater updater = new RecipeUpdater();
			updater.RemoveAll();
			updater.LoadFromXml();

			context = new DeliciousDataContext();
		}

		DeliciousDataContext context;

	}
}

#endif
#endregion
